Hi All,

Sorry I do not recall who asked the original question about trig 
functions but as I recall it he wanted something for use with 5.3 and 
Fivewin.

Following is a repeat of an article I wrote some time ago for the 
Clippings magazine of the now defunct Clipper User Group, Melbourne 
(Australia).

It should give someone some ideas, if only how poorly I program <g>.

Regards,

Ross McKenzie
Melbourne Australia



*****************************
SOME TRANSCENDENTAL FUNCTIONS 
Ross McKenzie

This article has been prompted by some very public begging
by David Peterson. No apologies are offered if the work is
detected as being Summer '87ish. It is. But don't worry,
you can rework them and resubmit them in next month's
Clippings!!!

Some years ago I had the craving to work with some heavenly
bodies .... astronomical that is :-^). I needed to
calculate the arctangent of a number associated with some
solar radiation data and found that there was a sad lack of
precanned Clipper functions to suit. Whilst some trig functions
had been published, e.g by Straley, they had limitations on
their input ranges or had inherent errors. So I massaged
them and the following code resulted.

Polynomial approximations can be used to achieve many
solutions with quite high orders of accuracy as these
examples demonstrate. Have you improved on someone's code
lately? Why not submit it. At least it will keep David's 
begging bowl hidden.

-------------------------
Function arcTan

* Returns the arcTan, expressed in Radians, of a number.
* Written by Ross McKenzie for the CUG(M), Melbourne, Australia
* The polynomial coefficients are drawn from the work of 
* B. Carlson & M Goldstein, Rational approximations of functions,
* Los Alamos Scientific Laboratory, New Mexico, 1953.
*
* The error ABS(e(arg)) <= 2*10^(-8)
*

parameter nTan_work
private nTan_x, nTan_squ, nTan_sign, lTan_toobig

nPI = 3.141592654
nRad = 57.29577951

set decimal to 12

if nTan_work < 0
   nTan_sign = -1
else
   nTan_sign = 1
endif

nTan_work = ABS(nTan_work)

if nTan_work > 1
   nTan_work = 1/nTan_work
   lTan_toobig = .t.
else
   lTan_toobig = .f.
endif

nTan_x = nTan_work
nTan_squ = nTan_work*nTan_work

nTan_work = -0.0161657367 +0.0028662257*nTan_squ
nTan_work =  0.0429096138 + nTan_work*nTan_squ
nTan_work = -0.0752896400 + nTan_work*nTan_squ
nTan_work =  0.1065626393 + nTan_work*nTan_squ
nTan_work = -0.1420889944 + nTan_work*nTan_squ
nTan_work =  0.1999355085 + nTan_work*nTan_squ
nTan_work = -0.3333314528 + nTan_work*nTan_squ
nTan_work =  1 + nTan_work*nTan_squ
nTan_work =  nTan_work*nTan_x

if lTan_toobig
   retval = nTan_sign*((nPI/2) - nTan_work)
else
   retval = ntan_sign*nTan_work
endif

return (retval)
**********************

Function Cosine

* Returns the Cosine of an angle parsed in either Degrees or Radians.  
* 
* Originally written by ESSOR MASO for Straley's 1988 Clipper Book 
* "Programming in Clipper", second edition.  
* Modified by Ross McKenzie to correct errors in handling + and - 
* 360 degrees and to remove input range limits.  
* 
* The error ABS(e(arg)) <= 2*10^(-9); the value 2 is returned if 
* either the argument or the D or R is missing.  
* 
* Correct usage : 
*                 @ x,y GET nD_ANGLE 
*                 read 
*                 nDangle = MOD(nD_angle, 360) 
*                 nRangle = nDangle*nPi/180 
*                 nCos_d = Cosine(nDangle,'D') 
*                 nCos_r = Cosine(nRangle,'R') 
*
* National Bureau of Standards Handbook of Mathematical Functions 
* (available at the library of the Bureau of Meteorology 
* 150 Lonsdale Street, Melbourne)

parameters nCos_work, cCos_unit

if pcount() < 2
   return(2)
endif
	
set decimal to 10

private nCos_int, nCos_squ

lCos_sign = nCos_work < 0
lDegrees = (upper(cCos_unit) = 'D')
	
nCos_work = abs(nCos_work)
	
if lDegrees
   * convert to radian units
   **************************
   nCos_work = nCos_work*1.5707963268/90
endif
	
nCos_int  = MOD(INT(nCos_work/1.5707963268),4)
nCos_work = MOD(nCos_work,1.5707963268)

if nCos_int = 1 .OR. nCos_int = 3
   nCos_work = 1.5707963268 - nCos_work
endif

store nCos_work*nCos_work to nCos_work,nCos_squ
nCos_work =  0.0000247609 -.0000002605*nCos_squ
nCos_work = -0.0013888397 + nCos_work*nCos_squ
nCos_work =  0.0416666418 + nCos_work*nCos_squ
nCos_work = -0.4999999963 + nCos_work*nCos_squ
nCos_work =  ROUNDTHEM( 1 + nCos_work*nCos_squ,8)

if nCos_int = 1.OR.nCos_int = 2
   nCos_work = -nCos_work
endif

return (nCos_work)
********************

Function Sine

* Returns the Sine of an angle supplied in either Degrees or Radians.
*
* Originally written by ESSOR MASO for Straley's 1988 Clipper Book
* "Programming in Clipper", second edition.
* Modified by Ross McKenzie to correct errors in handling + and - 
* 360 degrees and to remove input range limits.
* 
* The error ABS(e(arg)) is <= 2*10^(-9).
*
* The value 2 is returned if either the argument or the D or R is 
missing.
*
* Correct usage :
*                   @ x,y GET nD_ANGLE
*                   Read
*                   nDangle = MOD(nD_angle, 360)
*                   nRangle = nDangle*nPi/180
*                   nSin_d  = Sine(nDangle,'D')
*                   nSin_r  = Sine(nRangle,'R')
*
* National Bureau of Standards Handbook of Mathematical Functions
* (available at the library of the Bureau of Meteorology
* 150 Lonsdale Street, Melbourne)
   
parameters nSin_work, cSin_unit

if pcount() < 2
   return(2)
endif

set decimal to 10

lDegrees = (upper(cSin_unit) = 'D')
	
return (Cosine(iif(lDegrees,90,1.5707963268) - nSin_work, cSin_unit))
********************

Function Tangent

* Returns the Tangent of an angle supplied in either Degrees or Radians.
* Written by Ross McKenzie for the CUG(M), Melbourne, Australia
*
*	tan(n) = opp/adj = sine(n)/cosine(n)

parameter nAng_work, cAng_unit

nAdj = Cosine(nAng_work,cAng_unit)

if nAdj = 0.0
   * replace with small number to avoid divide by zero errors
   nAdj = 0.0000000017
endif

return (Sine(nAng_work,cAng_unit)/nAdj)
**********************

FUNCTION Roundthem
* Straley's code

PARAMETERS nThe_numb, nNo_places

set decimal to nNo_places

RETURN (iif(nThe_numb < 0, INT(nThe_numb * 10^nNo_places -0.5),;
               INT(nThe_numb*10^nNo_places +0.5))/10^nNo_places)
********************
